﻿'use strict';

var _ = require('underscore');
var fs = require('fs');
var async = require('async');
var path = require('path');

function objectMerge( obj1, obj2 ) {
	
	for (var p in obj2) {
		try {
			if ( obj2[p].constructor == Object ) {
				obj1[p] = objectMerge(obj1[p], obj2[p]);
			} else {
				obj1[p] = obj2[p];
			}
		} catch(e) {
			obj1[p] = obj2[p];
		}
	}
	return obj1;
	
}

module.exports.Engine = function( template, viewsPath, encoding ) {
	
	this.valid = function() {
		if ( this.viewsPath === undefined || this.viewsPath == null ) return false;
		if ( this.template === undefined || this.template == null ) return false;
		if ( !fs.existsSync( this.template ) ) return false;
		return true;
	};
	
	this.setupApp = function( app ) {
		
		if ( !this.valid() ) {
			console.error( 'Invalid Engine object' );
			return;
		}
		
		app.engine( 'use', function( file, options, callback ) {
			
			options = objectMerge( this.defaultOptions, options );
			
			async.waterfall( [
				
				function( callback ) {
					
					if ( options.include === undefined ) options.include = {};
					
					options._ = {
						file: this.file,
						template: this.template,
						fileExtension: '.use',
						fileBasename: path.basename( this.file, '.use' ),
						folderName: path.dirname( this.file ),
						includes: this.includes,
						template: this.template,
						encoding: this.encoding
					};
					
					callback( null, options );
					
				}.bind( {
					options: options,
					file: file,
					includes: this.templateIncludes,
					template: this.template,
					encoding: this.encoding
				} ),
				
				function( options, callback ) {
					
					try {
						
						for ( var includeid in options._.includes ) {
							
							var includename = options._.includes[ includeid ];
							
							var includePath = path.format({
								dir: options._.folderName,
								base: options._.fileBasename + includename + options._.fileExtension
							});
							
							if ( fs.existsSync( includePath ) ) {
								var includeFileContent = fs.readFileSync( includePath, options._.encoding );
								console.log(options._.encoding);
								var compiledInclude = _.template( includeFileContent );
								options.include[ includeid ] = compiledInclude( options );
							} else {
								options.include[ includeid ] = '';
							}
							
						}
						
						callback( null, options );
						
					} catch (err) {
						callback( err );
					}	
					
				},
				
				function( options, callback ) {
					
					try {
						
						var templateFileContent = fs.readFileSync( options._.template, options._.encoding );
						var compiledTemplate = _.template( templateFileContent );
						var finalContent = compiledTemplate( options );
						
						callback( null, finalContent );
						
					} catch (err) {
						callback( err );
					}					
					
				}
				
			], callback);
			
		}.bind( this ) );
		
		app.set( 'views', this.viewsPath );
		app.set( 'view engine', 'use' );
		
	};
	
	this.encoding = encoding === undefined ? 'utf-8' : encoding;
	this.template = template;
	this.templateIncludes = {};
	this.defaultOptions = {};
	this.viewsPath = viewsPath;
	
};

/*

var mainTemplatePath = '';
var encoding = 'utf-8';

function hasValidMainTemplate( callback ) {
	
	fs.exists( mainTemplatePath, function( exists ) {
		return callback( exists );
	} );
	
}

module.exports.setMainTemplatePath = function( path ) {
	mainTemplatePath = path;
};

module.exports.setEncoding = function( e ) {
	encoding = e;
};

module.exports.setupApp = function( app, views_path ) {
	
	app.engine( 'use', function( filePath, options, callback ) {
		
		async.parallel([
			function( callback ) {
				
				if ( options.page === undefined ) options.page = {};
				if ( options.page.include === undefined ) options.page.include = {};
				if ( options._includes === undefined ) options._includes = {};
				options.page.path = filePath;
				
				for ( var includeid in options._includes ) {
					
					var includeFileContent = fs.readFileSync( options._includes[ includeid ], encoding );
					var compiledInclude = _.template( includeFileContent );
					options.page.include[ includeid ] = compiledInclude( options );
					
				}
				
				var pageData = fs.readFileSync( filePath, encoding );
				var compiledPage = _.template( pageData );
				options.page.content = compiledPage( options );
				
				hasValidMainTemplate( function( exists ) {
					
					if (exists) {
					
						var templateData = fs.readFileSync( mainTemplatePath, encoding );
						var compiledTemplate = _.template( templateData );
						var rendered = compiledTemplate( options );
						
						callback( null, rendered );
						
					} else {
						
						var compiled = _.template( pageData );
						var rendered = compiled( options );
						
						callback( null, rendered );
						
					}
					
				} );
				
				return;
				
			}
		], function(err, datas) {
			callback(err, datas[0]);
		});
		
	} );
	
	app.set( 'views', views_path );
	app.set( 'view engine', 'use' );
	
}

*/